;*****************************************************************************
;*                                                                           *
;*                             ST6 EXAMPLE 1                                 *
;*                                                                           *
;*****************************************************************************

;|===========================================================================|
;| Assembly method without linker : ast6 -l -s -x -m -e example1.asm         |
;|===========================================================================|

;|===========================================================================|
;|                          Assembly directives                              |
;|===========================================================================|

	 .title "ST6 clignote"    ; This directive defines a new title to be
				  ; in the page header of the output listing.

	 .vers "st6225"
	 .romsize 4
	 .w_on                    ; utilisation des Labels
;*****************************************************************************
;*                              data space                                   *
;*****************************************************************************

;|===========================================================================|
;|                           data registers                                  |
;|===========================================================================|

	 .input "6215_reg.asm"

;|===========================================================================|
;|                            constants definition                           |
;|===========================================================================|

heud     .def 084h       
heuu     .def 085h       
mind     .def 086h
minu     .def 087h   
secd     .def 088h
secu     .def 089h     
cend     .def 08Ah
cenu     .def 08Bh 

compt    .def 08Ch
nbti     .def 08Dh               ;nbr de temps intermediaire
cmptnbti .def 08Eh               ;compteur de tps intermediaire
phase    .def 08Fh               ;gere les differentes phases: init, debut, ..
deuxsec  .def 090h

interm1  .def 0A0h               ;donnees pour le tps intermedaire no1 (a0-a7)
interm2  .def 0A8h               ;donnees pour le tps intermedaire no2 (a8-af)
interm3  .def 0B0h               ;donnees pour le tps intermedaire no1 (b0-b7)
interm4  .def 0b8h               ;donnees pour le tps intermedaire no1 (b8-bf)

debut    .equ 0                  ;affiche EPRAT
choix    .equ 1                  ;du nbr de tps inter
chrono   .equ 3                  ;chronometre
fin      .equ 4                  ;affichage des temps

;*****************************************************************************
;*                             PROGRAM SPACE                                 *
;*****************************************************************************

;|===========================================================================|
;|                             main program                                  |
;|===========================================================================|


	 .org 080h
chaine1  .asciz "Electronique"
chaine2  .asciz "Pratique (1996)"
chaine3  .asciz "Attente..."
chaine4  .asciz "Temps inter:"

reset    ldi wdr,0feh                   ; Load the watchdog.
	 call init                      ;initialise les ports d'entree sortie 
					; et les variables
	 call LCDInit                   ; initialise l'afficheur
	 call Eprat                     ; affiche presentation
	 
	 ldi ior,10h                    ; autorise les interruptions.
					; front descendants pour inter boutons 
	 reti                           ; fin du reset
	 nop

phase1   ldi wdr,0feh                   ; chien de garde   
	 ld a,phase
	 cpi a,choix                    ; attend une touche (interruption)
	 jrnz phase1
	 
	 call LCDClear                  ;ecran de presentation phase 2
	 ldi x,1                        ;"temps interme"         
	 ldi a,1
	 call LCDPos
	 ldi drwr,chaine4.w
	 ldi a,chaine4.d
	 ld x,a
	 call LCDWstrg
	 
	 ldi x,7                        
	 ldi a,2                        
	 call LCDPos
	 ld a,nbti
	 call LCDWnum 

phase2   ldi wdr,0feh                   ; chien de garde
	 ld a,phase                     ; l'utilisateur choisi le nbr de tps inter
	 cpi a,chrono            
	 jrnz phase2
	 
	 
loop     call LCDClear
	 ldi x,3                        ;"attente"                
	 ldi a,1
	 call LCDPos
	 ldi drwr,chaine3.w
	 ldi a,chaine3.d
	 ld x,a
	 call LCDWstrg
	  
	 ldi heud,0h
	 ldi heuu,0h  
	 
	 ldi mind,0h
	 ldi minu,0h    

	 ldi secd,0h
	 ldi secu,0h              
	 
	 ldi cend,0h
	 ldi cenu,0h         
	 
	 ldi compt,0                    ; chronometre en tant que tel ..
	 ldi tcr,0feh           
	 clr cmptnbti                   ; temps intermediaire 0
	 
phase3   ldi wdr,0feh                   ; chien de garde
	 ld a,phase                         
	 cpi a,fin           
	 jrnz phase3
	 call LCDClear                  ; l'utilisateur regarde ses tmps inter
	 call affich
	 
phase4   ldi wdr,0feh                   ; chien de garde
	 ld a,phase                     ; l'utilisateur desire recommencer
	 cpi a,chrono           
	 jrnz phase4                    
	 
	 jp loop
	 
;*****************************************************************************
;*                     Routine bouttons poussoir                             *
;*****************************************************************************

bouton   ld a,phase
	 jrz deb1                       ;evite l'overflow
    
	 cpi a,choix
	 jrz chx1                       ;evite l'overflow
	 cpi a,chrono
	 jrz chro1                      ;evite l'overflow
	 cpi a,fin
	 jrz fin11                      ;evite l'overflow
	 jp finbout
deb1     jp debut1
chx1     jp choix1
chro1    jp chrono1
fin11    jp fin1

debut1   ld a,drb
	 andi a,4+8+16                  ;teste si un bouton est enfonce 
					;(inter sur front descendant)
	 cpi a,4+8+16                   ;tout les boutons relaches ?
	 jrz finbou1                    ;evite l'overflow
	 ldi phase,choix                ;bouton actione=>phase choix
	 jp finbout                     ;evite l'overflow

finbou1  jp finbout                     

choix1   ld a,drb
	 andi a,4                       ;bouton + ?
	 ;cpi a,4    
	 jrnz choix2                     ;bouton + relache
		  
	 ld a,nbti                      ;bouton + enfonce
	 cpi a,4                        ;deja 4 tmps inter ? 
	 jrz finbou1                    ;oui, evite l'overflow
	 inc a                          ;non=> on ajoute un temps inter
	 ld nbti,a
	 jp choix3

choix2   ld a,drb                       ;bouton val ?
	 andi a,8                       
	 ;cpi a,8                        
	 jrnz choix4                     ;bouton val relache
	 ldi phase,chrono
	 jp finbout                    
finbou2  jp finbout                     ;evite l'overflow
choix4   ld a,drb                       ;bouton - ?
	 andi a,16      
	 ;cpi a,16
	 jrnz finbou2                   ;bouton - relache, evite l'overflow
	 ld a,nbti
	 jrz finbou2                    ;deja 0 tmps inter?, evite l'overflow
	 dec a
	 ld nbti,a
	 jp choix3
				
choix3   call LCDClear                  ;affiche l'ecran du nouveau choix
	 ldi x,1                        ;"temps inter"         
	 ldi a,1
	 call LCDPos
	 ldi drwr,chaine4.w
	 ldi a,chaine4.d
	 ld x,a
	 call LCDWstrg
	 
	 ldi x,7
	 ldi a,2                         
	 call LCDPos
	 ld a,nbti
	 call LCDWnum 
finbou3  jp finbout

chrono1  
	 ld a,tscr                      ;timer deja parti ?
	 andi a,00001000b               ;test du bit PSI
	 jrnz chrono2                   ;oui ...                  
         ldi tscr,01011111b             ;non=> demarre le timer
         
         clr deuxsec
         call LCDClear                  ;efface l'ecran 
	 jp finbout
chrono2  ld a,secd
	 jrnz chrono3                   
	 ld a,secu
	 jrnz chrono3
	 ld a,cend
	 jrnz chrono3
	 ld a,cenu
	 jrnz chrono3
	 jp finbout    
chro8    jp chrono8                 
finbou5  jp finbout
chrono3  ld a,deuxsec
         cpi a,2
         jrnz finbou5                    ;pas assez de temps entre 2 passages
         
         ld a,cmptnbti
         cp a,nbti
         jrz chro8
         clr deuxsec
         inc a                           ;ajoute un tmps inter
         ld cmptnbti,a
         ldi v,8                         ;longueur de la copie : 8
         ldi x,084h                      ;debut de la copie
         cpi a,1
         jrnz chrono4
         ldi y,interm1                   ;enregistre le tmps inter1
         call copie
         jp chrono7
chrono4  cpi a,2                         ;enregistre le tmps inter2
         jrnz chrono5
         ldi y,interm2
         call copie
         jp chrono7
chrono5  cpi a,3                         ;enregistre le tmps inter3
         jrnz chrono6
         ldi y,interm3
         call copie
         jp chrono7
chrono6  cpi a,4                         ;enregistre le tmps inter4
         jrnz chrono7
         ldi y,interm4
         call copie

chrono7  call affichT
         jp finbout
                  
chrono8  ldi tscr,01010111b             ;stoppe le timer    
	 ldi cmptnbti,0        
         ldi phase,fin
	 jp finbout 
	 
fin1     ld a,drb                       ;bouton val ?
	 andi a,8                       
	 jrnz fin2                      ;bouton val relache
         ldi phase,chrono               ;bouton val actione=>phase choix
         jp finbout 
fin2     ld a,drb
	 andi a,4                       ;bouton + ?
	 jrnz fin3                      ;bouton + relache
         
         ld a,cmptnbti
         cp a,nbti
         jrz finbou4            
         inc a
         ld cmptnbti,a
         jp fin4
finbou4  jp finbout
fin3     ld a,drb                       ;bouton - ?
	 andi a,16  
         jrnz finbou4 
         ld a,cmptnbti
         cpi a,0
         jrz finbou4
         dec a
         ld cmptnbti,a
fin4     call affichT
  
finbout  reti


;|---------------------------------------------------------------------------|
;|                            fonctions                                      |
;|---------------------------------------------------------------------------|
; cette fonction copie le bloc de donnee commencant a l'adresse x vers le bloc
; commencant a l'adresse y. La longueur du bloc copie est contenu dans v.
copie    ld a,(x)
         ld (y),a
         inc x
         inc y
         dec v
         jrnz copie
         ret	 

;|---------------------------------------------------------------------------|
;|                            interruption Timer                             |
;|---------------------------------------------------------------------------|

timer    ld A,compt
	 inc A
	 cpi  A,03h
	 jrz plus1
	 ldi tcr,21
	 jp ok
plus1    ldi tcr,22
	 clr A                          
ok       res 7,tscr                     ;autorise les interruptions
	 ld compt,A                                
	 ld A,cenu                      ;teste les centiemes                      
	 inc A
	 cpi  A,10                     
	 jrz suite1
	 ld cenu,A
	 jp suite
suite1   clr A                        
	 ld cenu,A
	 ld A,cend                      
	 inc A
	 cpi a,10
	 jrz suite2
	 ld cend,a
	 jp suite
suite2   clr a
	 ld cend,a
         
         ld a,deuxsec                   ; gere le temps minimum entre 2 tps inter
	 cpi a,2
         jrz suite8
         inc a 
         ld deuxsec,a
         
suite8   ld a,secu
	 inc a 
	 cpi a,10
	 jrz suite3
	 ld secu,a
	 jp suite
suite3   clr a
	 ld secu,a
	 ld a,secd
	 inc a
	 cpi a,6
	 jrz suite4
	 ld secd,a
	 jp suite
suite4   clr a
	 ld secd,a
	 ld a,minu
	 inc a 
	 cpi a,10
	 jrz suite5
	 ld minu,a
	 jp suite
suite5   clr a
	 ld minu,a
	 ld a,mind
	 inc a
	 cpi a,6
	 jrz suite6
	 ld mind,a
	 jp suite
suite6   clr a
	 ld mind,a
	 ld a,heuu
	 inc a 
	 cpi a,10
	 jrz suite7
	 ld heuu,a
	 jp suite
suite7   clr a
	 ld heuu,a
	 ld a,heud
	 inc a
	 ld heud,a
suite    call affich
	 reti                  
	 


;*****************************************************************************
;*                     Routine d'affichage temps inter                       *
;*****************************************************************************
; la vatiable cmptnbti contient le no du temps a afficher
affichT  ld a,cmptnbti
         jrnz aff_s
         ret    
aff_s    ldi x,0                         ;premiere ligne, caractere 1
         ldi a,1
         call LCDPos
         ldi a,'('
         call LCDWdata
         call LCDWait
         ld a,cmptnbti
         call LCDWnum
         ldi a,')'
         call LCDWdata
         call LCDWait
         ldi a,' '
         call LCDWdata
         call LCDWait
         
         ldi x,4
         ldi a,1
         call LCDPos
         ld a,cmptnbti
         cpi a,1
         jrnz affichT1
         ldi x,0a0h                      ;debut des donnees a afficher 
         jp aff_sui
affichT1 cpi a,2
         jrnz affichT2
         ldi x,0a8h                      ;debut des donnees a afficher 
         jp aff_sui
affichT2 cpi a,3
         jrnz affichT3
         ldi x,0b0h                      ;debut des donnees a afficher 
         jp aff_sui
affichT3 ldi x,0b8h                      ;debut des donnees a afficher 
aff_sui  call affich1
         ret
 
;*****************************************************************************
;*                     Routine d'affichage horloge                           *
;*****************************************************************************
affich  ldi x,4                 ;positionne le curseur
	ldi a,2
        call LCDPos

	ldi x,084h              ;debut des donnees a afficher 
	
affich1	ld a,(x)                ;heures
	call LCDWnum
	inc x
	ld a,(x)
	call LCDWnum
	ldi a,03ah
	call LCDWdata      
	inc x

	ld a,(x)
	call LCDWnum            ;minutes
	inc x
	ld a,(x)
	call LCDWnum
	ldi a,03ah
	call LCDWdata
	inc x


	ld a,(x)                ;secondes
	call LCDWnum
	inc x
	ld a,(x)
	call LCDWnum
	ldi a,03ah
	call LCDWdata      
	inc x
		
	ld a,(x)                ;centieme
	call LCDWnum
	inc x
	ld a,(x)
	call LCDWnum
	ret

;*****************************************************************************
;*                     Initialisation                                        *
;*****************************************************************************
init     clr dra                ;port en sortie
	 ldi ddra,0ffh
	 ldi ora,0ffh           ;Push-Pull ouvert
	 
	 clr drb
	 ldi ddrb,03h           ; port b bit 0 et 1 en sortie LCD, 
				; bit 2,3,4,entree bouton poussoir+interruption
				; et resistance de rappel
				; bit 2: plus, bit 3:valide, bit4:moins
	 ldi orb,0ffh 

	 clr drc                ; port c, temps inter:entree + interr+ R rappel 
	 clr ddrc
	 ldi orc,0ffh

	 clr nbti               ;0 temps intermediaire
	 clr cmptnbti
         clr deuxsec            ;deuxsec <=0 (pour eviter les declenchements hatifs)
	 ldi phase,debut         ;phase initiale, demande nombre de tps inter..
	 ldi tscr,01010111b      ; initialise le timer 
	 ret
	
	
	.input "lcd.asm"                        ;routine pour LCD.                    



;*****************************************************************************
;*                     affiche la presentation                               *
;*****************************************************************************
Eprat    ldi x,1                ;"Electronique"                
	 ldi a,1
	 call LCDPos
	 ldi drwr,chaine1.w
	 ldi a,chaine1.d
	 ld x,a
	 call LCDWstrg 
	 
	 ldi x,0                ;"Pratique (1996)"                
	 ldi a,2
	 call LCDPos
	 ldi drwr,chaine2.w
	 ldi a,chaine2.d
	 ld x,a
	 call LCDWstrg 
	 ret;         
		
	 
;*****************************************************************************
;*                     Restart and interrupt Vectors                         *
;*****************************************************************************
	 
	.org 0ff0h

	nop                     ;it ADC
	reti

	.org 0ff2h              ;it timer
	jp timer

	.org 0ff4h              ;port b et c
	jp bouton

	.org 0ff6h              ;port a
	nop
	reti

	.org 0ffch              ;nmi
	nop
	reti

	.org 0ffeh              ;reset
	jp reset
